/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.control.activity.task;

import org.objectweb.dream.control.activity.scheduler.Scheduler;
import org.objectweb.dream.control.activity.task.thread.BasicThreadPoolTask;
import org.objectweb.dream.control.activity.task.thread.ThreadPoolAttributeController;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;

/**
 * Provides utility functions to handle tasks.
 */
public final class Util
{

  private Util()
  {
  }

  /**
   * Creates a task.
   * 
   * @param task an object implementing the <code>Task</code> interface.
   * @param controllerDesc the name of the controller description.
   * @return a component implementing the <code>Task</code> interface.
   * @throws InstantiationException if an error occurs while creating the task.
   */
  public static Component createTask(Task task, String controllerDesc)
      throws InstantiationException
  {
    Component bootstrap = org.objectweb.fractal.api.Fractal
        .getBootstrapComponent();
    TypeFactory typeFactory = null;
    GenericFactory genericFactory = null;
    try
    {
      genericFactory = Fractal.getGenericFactory(bootstrap);
      typeFactory = Fractal.getTypeFactory(bootstrap);
    }
    catch (NoSuchInterfaceException e1)
    {
      Error.bug(null, e1);
    }

    // create interface type
    // The task server interface type
    InterfaceType taskItfType = typeFactory.createFcItfType(Task.ITF_NAME,
        Task.class.getName(), false, false, false);

    // The scheduler client interface type
    InterfaceType schedulerItfType = typeFactory.createFcItfType(
        Scheduler.ITF_NAME, Scheduler.class.getName(), true, true, false);

    // create component type
    ComponentType taskType = typeFactory.createFcType(new InterfaceType[]{
        taskItfType, schedulerItfType});

    // instantiate components
    Component component = genericFactory.newFcInstance(taskType,
        controllerDesc, task);

    if (task instanceof NameController)
    {
      try
      {
        Fractal.getNameController(component).setFcName(
            ((NameController) task).getFcName());
      }
      catch (NoSuchInterfaceException e)
      {
        // ignore
      }
    }

    return component;
  }

  /**
   * Creates a scheduler.
   * 
   * @param scheduler the object implementing the <code>Scheduler</code>
   *          interface.
   * @param taskName the name of the thread pool task to create. May be
   *          <code>null</code>.
   * @return a component implementing the <code>Scheduler</code> interface.
   * @throws InstantiationException if an error occurs while creating the
   *           scheduler.
   */
  public static Component createScheduler(Scheduler scheduler, String taskName)
      throws InstantiationException
  {
    Component bootstrap = org.objectweb.fractal.api.Fractal
        .getBootstrapComponent();
    TypeFactory typeFactory = null;
    GenericFactory genericFactory = null;
    try
    {
      genericFactory = Fractal.getGenericFactory(bootstrap);
      typeFactory = Fractal.getTypeFactory(bootstrap);
    }
    catch (NoSuchInterfaceException e1)
    {
      Error.bug(null, e1);
    }

    // create interface type
    // The Scheduler server interface
    InterfaceType schedulerItfType = typeFactory.createFcItfType(
        Scheduler.ITF_NAME, Scheduler.class.getName(), false, false, false);

    // The task client collection interface
    InterfaceType taskItfType = typeFactory.createFcItfType(Task.ITF_NAME,
        Task.class.getName(), true, false, true);

    // create component type
    ComponentType taskType = typeFactory.createFcType(new InterfaceType[]{
        schedulerItfType, taskItfType});

    // instantiate components
    Component component = genericFactory.newFcInstance(taskType,
        "taskPrimitive", scheduler);

    if (taskName != null)
    {
      try
      {
        Fractal.getNameController(component).setFcName(taskName);
      }
      catch (NoSuchInterfaceException e)
      {
        // ignore.
      }
    }

    return component;

  }

  /**
   * Creates a thread pool task.
   * 
   * @param taskName the name of the thread pool task to create. May be
   *          <code>null</code>.
   * @return a component interface of a thread pool task.
   * @throws InstantiationException if an error occurs while creating the task.
   */
  public static Component createThreadPoolTask(String taskName)
      throws InstantiationException
  {
    Component bootstrap = org.objectweb.fractal.api.Fractal
        .getBootstrapComponent();
    TypeFactory typeFactory = null;
    GenericFactory genericFactory = null;
    try
    {
      genericFactory = Fractal.getGenericFactory(bootstrap);
      typeFactory = Fractal.getTypeFactory(bootstrap);
    }
    catch (NoSuchInterfaceException e1)
    {
      Error.bug(null, e1);
    }

    // create interface type
    // The task server interface type
    InterfaceType taskItfType = typeFactory.createFcItfType(Task.ITF_NAME,
        Task.class.getName(), false, false, false);

    // The scheduler client interface type
    InterfaceType schedulerItfType = typeFactory.createFcItfType(
        Scheduler.ITF_NAME, Scheduler.class.getName(), true, true, false);

    // The attribute controller interface type
    InterfaceType attributeControllerItfType = typeFactory.createFcItfType(
        "attribute-controller", ThreadPoolAttributeController.class.getName(),
        false, false, false);

    // create component type
    ComponentType taskType = typeFactory.createFcType(new InterfaceType[]{
        taskItfType, schedulerItfType, attributeControllerItfType});

    // instantiate components
    Component component = genericFactory.newFcInstance(taskType,
        "threadPoolPrimitive", new BasicThreadPoolTask());

    if (taskName != null)
    {
      try
      {
        Fractal.getNameController(component).setFcName(taskName);
      }
      catch (NoSuchInterfaceException e)
      {
        // ignore.
      }
    }

    return component;
  }
}