/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.control.logger;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.loader.Initializable;
import org.objectweb.fractal.julia.loader.Tree;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Level;
import org.objectweb.util.monolog.api.Logger;

/**
 * Basic implementation of logger controller. A <code>"monolog-conf-file"</code>
 * initialisation parameter can be set to specify the location of the monolog
 * configuration file.
 */
public abstract class BasicLoggerControllerMixin
    implements
      LoggerController,
      LoggerControllerRegister,
      Initializable
{

  String baseName;

  /** Map associating logger name and list of registred loggables */
  Map    registrations;
  /** Map associating logger name and logger instance */
  Map    loggers;

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * @see Initializable#initialize(org.objectweb.fractal.julia.loader.Tree)
   */
  public void initialize(final Tree args) throws Exception
  {
    _super_initialize(args);
    for (int i = 0; i < args.getSize(); ++i)
    {
      Tree arg = args.getSubTree(i);
      if (arg.getSize() == 2 && arg.getSubTree(0).equals("monolog-conf-file"))
      {
        String monologConfFile = arg.getSubTree(1).toString();
        if (Monolog.monologFactory == Monolog.getDefaultMonologFactory())
        {
          Monolog.getMonologFactory(monologConfFile);
        }
      }
    }
  }

  /**
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public void initFcController(InitializationContext ic)
      throws InstantiationException
  {
    _super_initFcController(ic);
    baseName = Util.getNextUnnamedBaseName();
    registrations = new HashMap();
    loggers = new HashMap();
  }

  /**
   * @see LoggerController#getBaseName()
   */
  public String getBaseName()
  {
    return baseName;
  }

  /**
   * @see LoggerController#setBaseName(String)
   */
  public void setBaseName(String name)
  {
    this.baseName = name;
    baseNameChanged();
  }

  /**
   * @see LoggerController#getLoggerLevel(String)
   */
  public Level getLoggerLevel(String loggerName)
  {
    Logger l = (Logger) loggers.get(loggerName);
    if (l == null)
    {
      return null;
    }
    return l.getCurrentLevel();
  }

  /**
   * @see LoggerController#setLoggerLevel(String, Level)
   */
  public void setLoggerLevel(String loggerName, Level level)
  {
    Logger l = (Logger) loggers.get(loggerName);
    if (l == null)
    {
      return;
    }
    l.setLevel(level);
  }

  /**
   * @see LoggerControllerRegister#register(String, Loggable)
   */
  public void register(String loggerName, Loggable loggable)
  {
    Set s = (Set) registrations.get(loggerName);
    if (s == null)
    {
      s = new HashSet();
      registrations.put(loggerName, s);
    }
    s.add(loggable);
    giveLogger(loggerName, loggable);
  }

  /**
   * @see LoggerControllerRegister#unregiser(String, Loggable)
   */
  public void unregiser(String loggerName, Loggable loggable)
  {
    Set s = (Set) registrations.get(loggerName);
    if (s == null)
    {
      return;
    }
    s.remove(loggable);
    if (s.isEmpty())
    {
      // no more registration for this logger, remove reference to it, for
      // garbage collector
      registrations.remove(loggerName);
      loggers.remove(loggerName);
    }
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  void baseNameChanged()
  {
    loggers.clear();
    Iterator iter = registrations.entrySet().iterator();
    while (iter.hasNext())
    {
      Map.Entry entry = (Map.Entry) iter.next();
      String loggerName = (String) entry.getKey();
      Set loggables = (Set) entry.getValue();
      Iterator iter2 = loggables.iterator();
      while (iter2.hasNext())
      {
        Loggable loggable = (Loggable) iter2.next();
        giveLogger(loggerName, loggable);
      }
    }
  }

  void giveLogger(String loggerName, Loggable loggable)
  {
    Logger logger = (Logger) loggers.get(loggerName);
    if (logger == null)
    {
      String name;
      if (loggerName == null)
      {
        name = baseName;
      }
      else
      {
        name = baseName + "." + loggerName;
      }
      logger = Monolog.monologFactory.getLogger(name);
      loggers.put(loggerName, logger);
    }
    loggable.setLogger(loggerName, logger);
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The
   * {@link org.objectweb.fractal.julia.Controller#initFcController(InitializationContext) initFcController}
   * method overriden by this mixin.
   * 
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public abstract void _super_initFcController(InitializationContext ic)
      throws InstantiationException;

  /**
   * The {@link Initializable#initialize(Tree) initialize}method overriden by
   * this mixin.
   * 
   * @see Initializable#initialize(Tree)
   */
  public abstract void _super_initialize(final Tree args) throws Exception;

}