/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message.manager;

import java.util.Iterator;

import org.objectweb.dream.message.AbstractExtensibleMessage;
import org.objectweb.dream.message.BasicChunkType;
import org.objectweb.dream.message.BasicExtensibleMessage;
import org.objectweb.dream.message.BasicExtensibleMessageNC;
import org.objectweb.dream.message.Chunk;
import org.objectweb.dream.message.ChunkAlreadyExistException;
import org.objectweb.dream.message.ChunkType;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageAlreadyExistException;
import org.objectweb.dream.message.MessageType;
import org.objectweb.dream.message.MessageTypeNC;

/**
 * Basic implementation of a message manager. This message manager doesn't pool
 * instances.
 */
public class BasicMessageManagerImpl
    implements
      MessageManager,
      MessageManagerAttributeController
{

  // the id of this message manager
  short id = -1;

  // ---------------------------------------------------------------------------
  // Implementation of the MessageManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageManager#createMessage(MessageType)
   */
  public Message createMessage(MessageType type) throws UnknownChunkTypeError
  {
    ExtensibleMessage msg;
    Iterator iterator;
    if (type instanceof MessageTypeNC)
    {
      BasicExtensibleMessageNC message = new BasicExtensibleMessageNC(id);
      msg = message;
      MessageTypeNC typeNC = (MessageTypeNC) type;

      // add named sub messages
      iterator = typeNC.getSubMessageNamesIterator();
      while (iterator.hasNext())
      {
        String msgName = (String) iterator.next();
        MessageType t = typeNC.getSubMessageType(msgName);
        try
        {
          message.addSubMessage(msgName, createMessage(t));
        }
        catch (MessageAlreadyExistException e)
        {
          // cannot happend
        }
      }

      iterator = typeNC.getUnnamedSubMessageTypesIterator();
    }
    else
    {
      msg = new BasicExtensibleMessage(id);
      iterator = type.getSubMessageTypesIterator();
    }
    // add unnamed sub messages
    while (iterator.hasNext())
    {
      MessageType t = (MessageType) iterator.next();
      msg.addSubMessage(createMessage(t));
    }

    // add chunks
    iterator = type.getChunkNamesIterator();
    while (iterator.hasNext())
    {
      String chunkName = (String) iterator.next();
      ChunkType t = type.getChunkType(chunkName);
      try
      {
        msg.addChunk(chunkName, t, createChunk(t));
      }
      catch (ChunkAlreadyExistException e)
      {
        // cannot happend
      }
    }
    return msg;
  }

  /**
   * @see MessageManager#deleteMessage(Message)
   */
  public void deleteMessage(Message message)
  {
    ((AbstractExtensibleMessage) message).decrementReferenceCounter();
  }

  /**
   * @see MessageManager#duplicateMessage(Message, boolean)
   */
  public Message duplicateMessage(Message message, boolean clone)
  {
    if (clone)
    {
      Message newInstance = createMessage(message.getMessageType());
      message.transfertChunkStates(newInstance);
      return newInstance;
    }
    else
    {
      ((AbstractExtensibleMessage) message).incrementReferenceCounter();
      return message;
    }
  }

  /**
   * @see MessageManager#createChunk(ChunkType)
   */
  public Object createChunk(ChunkType type) throws UnknownChunkTypeError
  {
    if (type instanceof BasicChunkType)
    {
      try
      {
        Chunk chunk = (Chunk) ((BasicChunkType) type).getChunkImpl()
            .newInstance();
        chunk.setMessageManagerId(id);
        return chunk;

      }
      catch (Exception e)
      {
        throw new InternalError("Unable to instanciate chunk implementation : "
            + e.getLocalizedMessage());
      }
    }
    else
    {
      throw new UnknownChunkTypeError(type);
    }
  }

  /**
   * @see MessageManager#deleteChunk(Object)
   */
  public void deleteChunk(Object chunk)
  {
    // Do nothing
  }

  /**
   * @see MessageManager#duplicateChunk(Object, boolean)
   */
  public Object duplicateChunk(Object chunk, boolean clone)
  {
    if (chunk instanceof Chunk)
    {
      Chunk c = (Chunk) chunk;
      Chunk newInstance = (Chunk) createChunk(c.getType());
      c.transfertState(newInstance);
      return c;
    }
    else
    {
      throw new InternalError("Unable to duplicate chunk : " + chunk);
    }
  }

  /**
   * @see MessageManager#getMessageManagerId()
   */
  public short getMessageManagerId()
  {
    return id;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the MessageManagerAttributeController
  // ---------------------------------------------------------------------------

  /**
   * @see MessageManagerAttributeController#getId()
   */
  public short getId()
  {
    return id;
  }

  /**
   * @see MessageManagerAttributeController#setId(short)
   */
  public void setId(short id)
  {
    this.id = id;
  }

}